/*
    Sjeng - a chess variants playing program
    Copyright (C) 2000 Gian-Carlo Pascutto

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
                                                          
    File: crazy.c
    Purpose: bughouse/crazyhouse specific functions                  

*/ 

#include <assert.h>
#include "sjeng.h"
#include "protos.h"
#include "extvars.h"

int holding[2][16];
int num_holding[2];
int GhostHolding[2][16];
int num_GhostHolding[2];

int drop_piece;

int white_hand_eval;
int black_hand_eval;

unsigned long hold_hash;

#define HHash(x,y)  (hold_hash ^= zobrist[(x)][(y)])

/* input example : holding [BPPP] [QR] */
/* based on db's parser */
void ProcessHoldings(char str[])
{
  int c, i, j, k;

  memset(holding, 0, sizeof(holding));
  hold_hash = 0xC0FFEE00;

  white_hand_eval = 0;
  black_hand_eval = 0;
  reset_ecache();
  
  num_holding[WHITE] = 0;
  num_holding[BLACK] = 0;


  //Add Ghost Pieces

   for (i=0; i<=1; i++)
	for (j=0; j<=15; j++)
	{
      if (GhostHolding[i][j])
	  {
        for (k=1; k<=GhostHolding[i][j]; k++)
		{ 
          holding[i][j]++;
	      num_holding[i]++;
	      HHash((j), holding[i][(j)]);
		}
	  }
	}


  i = 0;
  for(c = WHITE; c <= BLACK; c++) 
    {
      while(str[i++] != '[')
	if(str[i] == 0) return;
      
      while(str[i] != ']') {
	switch(str[i++]) {
	case 'p':
	case 'P':
	  holding[c][c == WHITE ? wpawn : bpawn]++;
	  num_holding[c]++;
	  HHash((c == WHITE ? wpawn : bpawn),
		holding[c][(c == WHITE ? wpawn : bpawn)]); 
	  break;
	case 'q':
	case 'Q':
	  holding[c][c == WHITE ? wqueen : bqueen]++;
	  num_holding[c]++;
	  HHash((c == WHITE ? wqueen : bqueen),
		holding[c][(c == WHITE ? wqueen : bqueen)]); 
	  break;
	case 'r':
	case 'R':
	  holding[c][c == WHITE ? wrook : brook]++;
	  num_holding[c]++;
	  HHash((c == WHITE ? wrook : brook),
		holding[c][(c == WHITE ? wrook : brook)]); 
	  break;
	case 'b':
	case 'B':
	  holding[c][c == WHITE ? wbishop : bbishop]++;
	  num_holding[c]++;
	  HHash((c == WHITE ? wbishop : bbishop),
		holding[c][(c == WHITE ? wbishop : bbishop)]); 
	  break;
	case 'n':
	case 'N':
	  holding[c][c == WHITE ? wknight : bknight]++;
	  num_holding[c]++;
	  HHash((c == WHITE ? wknight : bknight),
		holding[c][(c == WHITE ? wknight : bknight)]); 
	  break;
	default:
	  return;
	}
      }
    }
}


void ProcessGhostHoldings(char str[])
{
  int c, i,j,k;
  int OldGhostHolding[2][16];

  i = 0;

  //Ghost Pieces are in string same way like normal,
  // but are added into GhostHolding array.
  // Will be used later :
  // Pieces will be added into normal holding in some moment
  // and to be able recognize if move was played with ghost piece
  // will be checked if holding[Color][Piece]<=GhostHolding[Color][Piece].
  
  

  
  for (i=0; i<=1; i++)
	  for (j=0; j<=15; j++)
        OldGhostHolding[i][j]=GhostHolding[i][j];

  memset(GhostHolding, 0, sizeof(GhostHolding));
  num_GhostHolding[WHITE] = 0;
  num_GhostHolding[BLACK] = 0;

  i=0;
  for(c = WHITE; c <= BLACK; c++) 
    {
      while(str[i++] != '[')
	   if(str[i] == 0) goto LblRefreshHolding;
      
     while(str[i] != ']') {
	switch(str[i++]) {
	case 'p':
	case 'P':
	  GhostHolding[c][c == WHITE ? wpawn : bpawn]++;
	  num_GhostHolding[c]++;
	  break;
	case 'q':
	case 'Q':
	  GhostHolding[c][c == WHITE ? wqueen : bqueen]++;
	  num_GhostHolding[c]++;
	  break;
	case 'r':
	case 'R':
	  GhostHolding[c][c == WHITE ? wrook : brook]++;
	  num_GhostHolding[c]++;
	  break;
	case 'b':
	case 'B':
	  GhostHolding[c][c == WHITE ? wbishop : bbishop]++;
	  num_GhostHolding[c]++;
	  break;
	case 'n':
	case 'N':
	  GhostHolding[c][c == WHITE ? wknight : bknight]++;
	  num_GhostHolding[c]++;
	  break;
	default:
	  return;
	}
      }
    };

   LblRefreshHolding:
   
   for (i=0; i<=1; i++)
	for (j=0; j<=15; j++)
	{
      if (OldGhostHolding[i][j]<GhostHolding[i][j])
	  {
        while (OldGhostHolding[i][j]<GhostHolding[i][j])
		{
          holding[i][j]++;
	      num_holding[i]++;
	      HHash((j), holding[i][(j)]);
		  OldGhostHolding[i][j]++;
		}

	  }
	  else if (OldGhostHolding[i][j]>GhostHolding[i][j])
	  {
        while (OldGhostHolding[i][j]>GhostHolding[i][j])
		{
          holding[i][j]--;
	      num_holding[i]--;
	      HHash((j), holding[i][(j)]);
		  OldGhostHolding[i][j]--;
		}
	  }

	};


}


int text_to_piece(char txt, int who)
{
  switch(txt)
    {
    case 'p':
    case 'P':
      return (who == WHITE ? wpawn : bpawn);
    case 'b':
    case 'B':
      return (who == WHITE ? wbishop : bbishop);
    case 'n':
    case 'N':
      return (who == WHITE ? wknight : bknight);
    case 'r':
    case 'R':
      return (who == WHITE ? wrook : brook);
    case 'q':
    case 'Q':
      return (who == WHITE ? wqueen : bqueen);
    };

  return npiece;
}

int SwitchColor(int piece)
{
  int t[] = { 0, bpawn, wpawn, bknight, wknight, 0, 0, brook, wrook, bqueen, wqueen, bbishop, wbishop };

  assert(piece > frame  && piece < npiece);

  return(t[piece]);
}

int SwitchPromoted(int piece)
{
  int t[] = { 0, bpawn, wpawn, bpawn, wpawn, 0, 0, bpawn, wpawn, bpawn, wpawn, bpawn, wpawn };

  assert(piece > frame && piece < npiece);

  return(t[piece]);
}

void addHolding(int what, int who)
{

  if (Variant == Crazyhouse)
    {

      holding[who][what]++;
      
      num_holding[who]++;
      
      HHash(what, holding[who][what]);

    };

  if (who == WHITE)
    white_hand_eval += hand_value[what];
  else
    black_hand_eval += hand_value[what];

  Material += material[what];

  return;
    
}

void removeHolding(int what, int who)
{

  if (Variant == Crazyhouse)
    {

      assert(holding[who][what] > 0);
      assert(holding[who][what] < 10);	
      
      HHash(what, holding[who][what]);
      
      holding[who][what]--;
      
      num_holding[who]--;
      
    }

  if (who == WHITE)
  	white_hand_eval -= hand_value[what];
  else
    	black_hand_eval -= hand_value[what];

  Material -= material[what];

  return;

}

void DropaddHolding(int what, int who)
{
  holding[who][what]++;
  
  num_holding[who]++;
  
  HHash(what, holding[who][what]);
  
  if (who == WHITE)
    white_hand_eval += hand_value[what];
  else
    black_hand_eval += hand_value[what];

  Material += material[what];

  return;
}

void DropremoveHolding(int what, int who)
{
  assert(holding[who][what] > 0);

  assert(holding[who][what] < 10);	

  HHash(what, holding[who][what]);
  
  holding[who][what]--;
  
  num_holding[who]--;

  if (who == WHITE)
      white_hand_eval -= hand_value[what];
  else
      black_hand_eval -= hand_value[what];

  Material -= material[what];

  return;
}

void printHolding(void)
{

  printf("WP: %d WR: %d WB: %d WN: %d WQ: %d\n",
	 holding[WHITE][wpawn], holding[WHITE][wrook], 
	 holding[WHITE][wbishop],
	 holding[WHITE][wknight], holding[WHITE][wqueen]);
  
  printf("BP: %d BR: %d BB: %d BN: %d BQ: %d\n",
	 holding[BLACK][bpawn], holding[BLACK][brook], 
	 holding[BLACK][bbishop],
	 holding[BLACK][bknight], holding[BLACK][bqueen]);

}


void printGhostHoldingInfo(void)
{
  char HandWhite[STR_BUFF],HandBlack[STR_BUFF];

  if (GhostHolding[WHITE][wpawn]>0)
	sprintf(HandWhite,"P%d ",GhostHolding[WHITE][wpawn]);
  else 
	sprintf(HandWhite," ");
  if (GhostHolding[WHITE][wbishop]>0)
    sprintf(HandWhite,"%sB%d ",HandWhite,GhostHolding[WHITE][wbishop]);
  else 
	sprintf(HandWhite,"%s ",HandWhite);
  if (GhostHolding[WHITE][wknight]>0)
    sprintf(HandWhite,"%sN%d ",HandWhite,GhostHolding[WHITE][wknight]);
  else 
	sprintf(HandWhite,"%s ",HandWhite);
  if (GhostHolding[WHITE][wrook]>0) 
	sprintf(HandWhite,"%sR%d ",HandWhite,GhostHolding[WHITE][wrook]);
  else 
	sprintf(HandWhite,"%s ",HandWhite);
  if (GhostHolding[WHITE][wqueen]>0) 
	sprintf(HandWhite,"%sQ%d ",HandWhite,GhostHolding[WHITE][wqueen]);

  if (GhostHolding[BLACK][bpawn]>0) 
    sprintf(HandBlack,"P%d ",GhostHolding[BLACK][bpawn]);
  else 
	sprintf(HandBlack," ");
  if (GhostHolding[BLACK][bbishop]>0)
    sprintf(HandBlack,"%sB%d ",HandBlack,GhostHolding[BLACK][bbishop]);
  else 
	sprintf(HandBlack,"%s ",HandBlack);
  if (GhostHolding[BLACK][bknight]>0) 
    sprintf(HandBlack,"%sN%d ",HandBlack,GhostHolding[BLACK][bknight]);
  else 
	sprintf(HandBlack,"%s ",HandBlack);
  if (GhostHolding[BLACK][brook]>0) 
	sprintf(HandBlack,"%sR%d ",HandBlack,GhostHolding[BLACK][brook]);
  else 
	sprintf(HandBlack,"%s ",HandBlack);
  if (GhostHolding[BLACK][bqueen]>0) 
	sprintf(HandBlack,"%sQ%d ",HandBlack,GhostHolding[BLACK][bqueen]);
 

   if (!comp_color)
     printf("tellics ptell Ok, I now expect this pieces can coming : My OPPonent White [%s] Me black [%s].\n",HandWhite,HandBlack);
   else
	 printf("tellics ptell Ok, I now expect this pieces can coming : Me white [%s] My OPPonent black [%s].\n",HandWhite,HandBlack);
   
   /*
   if (!comp_color)
     printf("tellics whisper Ghost pieces i counting with: My OPPonent White [%s] Me black [%s].\n",HandWhite,HandBlack);
   else
	 printf("tellics whisper Ghost pieces i counting with: Me white [%s] My OPPonent black [%s].\n",HandWhite,HandBlack);
   */



}